import { calculateOffset, calculateMask, copyArrays, nextPowerOfTwo, SHIFT0, SHIFT1, SHIFT2, SHIFT3 } from "./Utils";
var BitSet = /** @class */ (function () {
    function BitSet(size) {
        if (size === void 0) { size = 0; }
        this.capacity = 0;
        this.internalSize = 0;
        this.grow(size);
    }
    BitSet.prototype.grow = function (size) {
        if (this.capacity <= size) {
            var capacity = nextPowerOfTwo(size);
            var p0 = calculateOffset(capacity, SHIFT1);
            var p1 = calculateOffset(capacity, SHIFT2);
            var p2 = calculateOffset(capacity, SHIFT3);
            this.layer0 = copyArrays(this.layer0, new Uint32Array(p0 + 1));
            this.layer1 = copyArrays(this.layer1, new Uint32Array(p1 + 1));
            this.layer2 = copyArrays(this.layer2, new Uint32Array(p2 + 1));
            this.layer3 = copyArrays(this.layer3, new Uint32Array(1));
            this.capacity = capacity;
        }
        if (this.internalSize < size) {
            this.internalSize = size;
        }
    };
    BitSet.prototype.byte = function (index, layer) {
        switch (layer) {
            case 0: return this.layer0[index];
            case 1: return this.layer1[index];
            case 2: return this.layer2[index];
            case 3: return this.layer3[index];
        }
    };
    BitSet.prototype.size = function () {
        return this.internalSize;
    };
    BitSet.prototype.add = function (index) {
        this.grow(index + 1);
        var p0 = calculateOffset(index, SHIFT1);
        var mask = calculateMask(index, SHIFT0);
        var value = this.layer0[p0];
        if ((value & mask) != 0) {
            return true;
        }
        this.layer0[p0] |= mask;
        if (value == 0) {
            this.add_slow(index);
        }
        return false;
    };
    BitSet.prototype.remove = function (index) {
        var p0 = calculateOffset(index, SHIFT1);
        var p1 = calculateOffset(index, SHIFT2);
        ;
        var p2 = calculateOffset(index, SHIFT3);
        var mask = calculateMask(index, SHIFT0);
        var mask1 = calculateMask(index, SHIFT1);
        var mask2 = calculateMask(index, SHIFT2);
        var mask3 = calculateMask(index, SHIFT3);
        if ((this.layer0[p0] & mask) == 0) {
            return false;
        }
        this.layer0[p0] &= ~mask;
        if (this.layer0[p0] != 0) {
            return true;
        }
        this.layer1[p1] &= ~mask1;
        if (this.layer1[p1] != 0) {
            return true;
        }
        this.layer2[p2] &= ~mask2;
        if (this.layer2[p2] != 0) {
            return true;
        }
        this.layer3[0] &= ~mask3;
        return true;
    };
    BitSet.prototype.contains = function (index) {
        var p0 = calculateOffset(index, SHIFT1);
        var mask = calculateMask(index, SHIFT0);
        return p0 < this.layer0.length && ((this.layer0[p0] & mask) != 0);
    };
    BitSet.prototype.add_slow = function (index) {
        var p1 = calculateOffset(index, SHIFT2);
        var p2 = calculateOffset(index, SHIFT3);
        this.layer1[p1] |= calculateMask(index, SHIFT1);
        this.layer2[p2] |= calculateMask(index, SHIFT2);
        this.layer3[0] |= calculateMask(index, SHIFT3);
    };
    return BitSet;
}());
export { BitSet };
//# sourceMappingURL=BitSet.js.map